@@ -1,19 +1,23 @@
-# Copyright (c) 2013, 2016 Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2013, 2019, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 import grt
 from workbench.log import log_warning, log_error
@@ -151,7 +155,7 @@ class UserHostPrivileges(object):
         """
         ret_val = 0
 
-        if self._privileges.has_key(level):
+        if level in self._privileges:
             new_value = 0
             
             # Gets the value of each privilege in privileges and appends its
@@ -171,20 +175,24 @@ class UserHostPrivileges(object):
         return ret_val
 
     def add_privileges(self, target, priv_list):
-
         # Converts the privilege lists to its binary representation
         priv_value = self.get_privilege_value(priv_list, target.get_target_type())
 
         # Creates the entry for the target if it doesn't exist
-        if not self._granted_privileges.has_key(target):
-            self._granted_privileges[target] = 0
+        target_to_use = None
+        for tgt in list(self._granted_privileges.keys()):
+            if tgt.matches(target):
+                target_to_use = tgt
 
+        if target_to_use is None:
+                self._granted_privileges[target] = 0
+                target_to_use = target
+        
         # Appends the new privileges to the existing ones
-        self._granted_privileges[target] |= priv_value
-
+        self._granted_privileges[target_to_use] |= priv_value
 
     def check_privileges(self, target, privileges):
-        for tgt in self._granted_privileges.keys():
+        for tgt in list(self._granted_privileges.keys()):
             if tgt.matches(target):
                 # Gets the included privileges
                 includes = self._granted_privileges[tgt] & privileges
@@ -229,7 +237,7 @@ class UserHostPrivileges(object):
             # On the indicated host
             result = self.context.ctrl_be.exec_query("SHOW GRANTS FOR `%s`@`%s`" % (self.user, host))
             
-            context = grt.modules.MySQLParserServices.createParserContext(self._character_sets, self._target_version, 'STRICT_TRANS_TABLES,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION', 1)
+            context = grt.modules.MySQLParserServices.createNewParserContext(self._character_sets, self._target_version, 'STRICT_TRANS_TABLES,NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION', 1)
             
             if result:
                 while result.nextRow():
@@ -245,7 +253,7 @@ class UserHostPrivileges(object):
                             target = None
             
                             # Search for an already existing target
-                            for tgt in self._granted_privileges.keys():
+                            for tgt in list(self._granted_privileges.keys()):
                                 if tgt.identical(target_string):
                                     target = tgt
             
@@ -261,7 +269,7 @@ class UserHostPrivileges(object):
                             self.add_privileges(target, priv_list)
                         else:
                             log_error('An error occurred parsing GRANT statement: %s\n -> %s\n' % (statement, grant_data['error']))
-                    except Exception, exc:
+                    except Exception as exc:
                         log_error('An error occurred parsing GRANT statement: %s\n -> %s\n' % (statement, exc))
 
             else:
